#ifndef __LAYERS_H__
#define __LAYERS_H__

#include <vstypes.h>

#define USE_CRC /* Currently implemented for layer 3 only */

#define         SBLIMIT                 32
#define         SCALE_BLOCK             12
#define         SSLIMIT                 18

#define         MPG_MD_STEREO           0
#define         MPG_MD_JOINT_STEREO     1
#define         MPG_MD_DUAL_CHANNEL     2
#define         MPG_MD_MONO             3

#define EXTRA_QUANT_BITS 9 /* for 32-bit quantization, must be at least 3 */
/*
  EXTRA_QUANT_BITS effect in layer 3 compliance (-20dB sin sweep):
   7:  -99.2dB (7.7684e-06)
   8:  -99.9dB (7.1876e-06)
   9: -100.1dB (6.9984e-06)
  10: -100.2dB (6.9218e-06)
  11: -100.2dB (6.9010e-06)
  12: -100.2dB (6.8991e-06)
 */

#ifndef ASM

#ifdef ALLOC_IN_X
#define ALLOC_MEM
#else
#define ALLOC_MEM __y
#endif

#define NEW_ALLOC
struct frame {
#ifdef NEW_ALLOC
    ALLOC_MEM const struct al_table * const *alloc;
#else
    ALLOC_MEM const struct al_table *alloc;
#endif
    int stereo;
    int jsbound;
    int single;
    int II_sblimit;
    int lsf;
    int mpeg25;
    int header_change;
    int lay;
    int error_protection;
    int bitrate_index;
    int sFreqIdx;
    int padding;
    int mode;
    int mode_ext;
    int framesize; /* computed framesize */
#ifdef USE_CRC
    unsigned short file_crc;
    unsigned short actual_crc;
#endif
    int reservoir_len;
};
#ifdef USE_CRC
u_int16 CalcCrc(register __i0 __y u_int16 *data, s_int16 len, u_int16 initial);
#endif/*USE_CRC*/


u_int16 get1bit(void);
auto void rewindbits(register __a1 int number_of_bits);
auto u_int16 getbits(register __a1 int);
auto void SetPointer(s_int16);

/* AUDIOBUFSIZE = 64 */
#define		AUDIOBUFSIZE		64
extern s_int16 pcm_sample[AUDIOBUFSIZE];
extern s_int16 pcm_point;

struct gr_info_s {
      s_int16 scfsi;
      u_int16 part2_3_length;
      u_int16 big_values;
      u_int16 scalefac_compress;
      u_int16 block_type;
      u_int16 mixed_block_flag;
      u_int16 table_select[3];
      u_int16 maxband[3];
      u_int16 maxbandl;
      u_int16 region1start;
      u_int16 region2start;
      u_int16 preflag;
      u_int16 scalefac_scale;
      u_int16 count1table_select;
      s_int16 full_gain[3];
      s_int16 pow2gain;
};

struct III_sideinfo {
  u_int16 main_data_begin;
  struct {
    struct gr_info_s gr[2];
  } ch[2];
};

void do_layer3(void);
auto void InitLayer3(void);
auto void SetLayer3Acc(u_int16 acc); /* 0 = worst, 3 = best */
void do_layer2(void);
void do_layer1(void);

/* Parameter 1 is in format 1.1.14 for range -2.0 .. 2.0 */
auto void synth(s_int16 *,s_int16,s_int16 *,s_int16 *);
auto void synth_mono(s_int16 *,s_int16 *,s_int16 *);
void InitDecode(void);


auto void synth_mono32(s_int32 *,s_int16 *,s_int16 *);
auto void synth32(s_int32 *bandPtr, s_int16 ch, s_int16 *out, s_int16 *pnt);



extern unsigned char *conv16to8;
extern const u_int16 freqs[16];
extern const char * const freqNames[9];
extern const u_int16 __y muls[64];  /* Values fit    0 .. 2.0     -> 0.1.15 */
extern const s_int16 mulmul[27];    /* Values fit -1.0 .. 1.0     -> 1.0.15 */
extern const s_int16 __y decwin[512+32];/* Values fit -32768..32767->1.15.0 */
extern const s_int16 __y decwin16[256+16]; /* Aligned to 16-word boundaries */
extern const s_int16 __y decwin16scaled[256+16];

/* LAYER2 */
#ifdef GRP_TAB_IN_X
/* Only allow packed version now.. */
extern const u_int16 grp_3tab[32];     /* used: 27 */
extern const u_int16 grp_5tab[128];     /* used: 125 */
extern const u_int16 grp_9tab[729];     /* used: 729 */
#else
extern const u_int16 __y grp_3tab[32]; /* used: 27 */
extern const u_int16 __y grp_5tab[128]; /* used: 125 */
extern const u_int16 __y grp_9tab[729]; /* used: 729 */
#endif

/* LAYER3 */
#define GAINPOW_SZ 64
extern const u_int16 __y gainpow2[GAINPOW_SZ];
//extern const u_int16 aa_ca[8],aa_cs[8];
extern const __y s_int16 aa_csca[2*8];
extern const s_int16 win[4][36];
extern const s_int16 win1[4][36];
extern const s_int16 tan1_1[2*16],tan2_1[2*16],tan1_2[2*16],tan2_2[2*16];
extern const s_int16 pow1_1[2][2*16],pow2_1[2][2*16],pow1_2[2][2*16],pow2_2[2][2*16];

#ifdef LIMITS_IN_Y
extern const __y u_int16 longLimit[9][23];
extern const __y u_int16 shortLimit[9][14];
#else
extern const u_int16 longLimit[9][23];
extern const u_int16 shortLimit[9][14];
#endif
extern const s_int16 mapbuf0[9][152];
extern const s_int16 mapbuf1[9][156];
extern const s_int16 mapbuf2[9][44];
extern const u_int16 n_slen2[512];
#ifdef ISLEN_IN_Y
extern const __y u_int16 i_slen2[256];
#else
extern const u_int16 i_slen2[256];
#endif

/* These may be used by layer 1 and 2 for other purposes.
   Size for each table is 1152. */
extern s_int32 hybridIn32[2][SBLIMIT][SSLIMIT];

/* For pow43 */
extern u_int16 const     pow43_1_mant[];
extern u_int16 const __y pow43_1_exp[];
extern u_int16 const     pow43_2_mant[];
extern u_int16 const __y pow43_2_exp[];
extern u_int16 const     pow43_4_mant[];
extern u_int16 const __y pow43_4_exp[];

/* For cos64, imdct36 & imdct12 */
extern s_int16 __y const cos64NewTab[];
extern s_int16 __y const cos36Tab[];
extern s_int16 __y const cos12Tab[];

/* For bit routines */
extern u_int16 __y * __y wordpointer;
extern u_int16 __y bitindex;
extern s_int16 __y bitsremaining;

/* Decode.c's buffer that must be converted with data */
#define SYNTH_BUF_SIZE 0x110
extern s_int32 synth_buffs_32[2][2][SYNTH_BUF_SIZE]; /* 2*1088w, aligned-32! */

#define FRAMESPACE (1792+512)

extern __y struct L123Y {
    u_int16 bsspace[FRAMESPACE/2]; /* 1152 words -- holds one frame */
    struct frame fr; /* 19 words */
    union {
	struct {
	    s_int16 smpb[2*SBLIMIT];
	    u_int16 balloc[2*SBLIMIT];
	    /* 128 */
	    //1299
	    u_int16 scale_index[2*SBLIMIT];
	    //1363
	} l1;
	struct {
	    u_int16 bit_alloc[2*SBLIMIT];
	    u_int16 scfsi_buf[2*SBLIMIT];
	    /* 128 */
	    u_int16 scale[192]; /*overlaps g_chn+g_bark */
	    //1491
	} l2;
	struct {
	    struct III_sideinfo sideinfo; /*105*/
	    /* 183 */
	    u_int16 scalefacs[2][39]; /* 78*/  /*overlaps g_chn+g_bark */
	    //1354
	} l3;
    } i;
} layer123y; //1491

extern s_int16 synth_temp[2*32];
#endif/*!ASM*/


#endif/* !__LAYERS_H__*/
