#ifndef __PARAMETRIC_H__
#define __PARAMETRIC_H__

#define PARAMETRIC_VERSION 0x0004

#define PLAYMODE_MONO_OUTPUT (1<<0)
#define PLAYMODE_PAUSE_ON    (1<<1)
#define PLAYMODE_VUMETER_ON  (1<<2)
#define PLAYMODE_ADMIXER_ON  (1<<3)
#define PLAYMODE_PCMMIXER_ON (1<<4)
#define PLAYMODE_EQ5_ON      (1<<5)
#define PLAYMODE_SPEEDSHIFTER_ON      (1<<6)

#define ADMIXER_RATEMASK    (3<<0)
#define ADMIXER_RATE192     (0<<0) /* 5MHz   */
#define ADMIXER_RATE96      (1<<0) /* 2.5MHz */
#define ADMIXER_RATE48      (2<<0) /* 1.25MHz*/
#define ADMIXER_RATE24      (3<<0) /* 0.6MHz */
#define ADMIXER_MODEMASK    (3<<2)
#define ADMIXER_MODESTEREO  (0<<2)
#define ADMIXER_MODEMONO    (1<<2)
#define ADMIXER_MODELEFT    (2<<2)
#define ADMIXER_MODERIGHT   (3<<2)

#define CFG1_NOWMA  (1<<15)
#define CFG1_NOAAC  (1<<14)
#define CFG1_NOMP3  (1<<13)
#define CFG1_NOFLAC (1<<12) /*To allow more memory to be used by user.*/
#define CFG1_PSNORMAL      (0<<6)
#define CFG1_PSDOWNSAMPLED (1<<6) /* PS in downsampled mode */
#define CFG1_PSOFF         (3<<6) /* no PS */
#define CFG1_SBRNORMAL      (0<<4)
#define CFG1_SBRNOIMPLICIT  (1<<4) /*default!*/
#define CFG1_SBRDOWNSAMPLED (2<<4) /*never upsample*/
#define CFG1_SBROFF         (3<<4) /*no SBR or PS */
#define CFG1_MP3_NOCRC      (1<<8) /*turn off CRC checking*/
#define CFG1_REVERB         (1<<0) /*for MIDI */

#ifdef ASM

#else/*ASM*/

#if PARAMETRIC_VERSION == 4 /*for vs1063*/
struct parametric {
    /* configs are not cleared between files */
    u_int32 chipID;     /*0x1e00/01 Initialized at reset for your convenience*/
    u_int16 version;    /*0x1e02 - structure version */
    /* config1  0x1e03 wamf ---C ppss RRRR */
    u_int16 config1;
    s_int16 playSpeed;  /*0x1e04 0,1 = normal speed, 2 = twice, etc. */
    u_int16 bitRatePer100; /*0x1e05 average bitrate / 100 */

    u_int16 endFillByte;   /*0x1e06 which byte value to send after file */
    s_int32 rateTune;      /*0x1e07..8 samplerate tune in +-2ppm steps. V4*/
    u_int16 playMode;      /*0x1e09 V4
			     bit 0 = mono output
			     bit 1 = pause on
			     bit 2 = VU meter enable
			     bit 3 = AD mixer enable
			     bit 4 = PCM mixer enable
			     bit 5 = EQ5 enable
			     bit 6 = SPEEDSHIFTER enable
			   */
    s_int32 sampleCounter; /*0x1e0a..b sample counter. V4*/
    u_int16 vuMeter;       /*0x1e0c VU meter result V4*/
    s_int16 adMixerGain;   /*0x1e0d AD mixer attenuation in 3dB steps -3..-31*/
    u_int16 adMixerConfig; /*0x1e0e AD mixer config
			     bits 5-4= AD mixer rate (0=192k,1=96k,2=48k,3=24k)
			     bits 7-6= AD mixer mode (0=stereo,1=mono,2=L,3=R)
			   */
    u_int16 pcmMixerRate;  /*0x1e0f PCM mixer sample rate (read when enabled)*/
    u_int16 pcmMixerFree;  /*0x1e10 PCM mixer FIFO free state */
    u_int16 pcmMixerVol;   /*0x1e11 PCM mixer volume 0..191 (-0.5dB steps) */
    u_int16 eq5Params[10]; /*0x1e12..0x1e1b 5-channel EQ parameters */
    u_int16 eq5Updated;    /*0x1e1c write as non-zero to recalculate filters.*/
    u_int16 speedShifter;  /*0x1e1d Speed shifter speed 0x4000 == 1.0x V4 */
    u_int16 earSpeakerLevel;  /*0x1e1e EarSpeaker level, 0 = off. V4*/
    /* sdiFree and audioFill are updated when WRAMADDR is written with
       values from 0xc0c0 to 0xc0ff. These translate to parametric
       stucture addresses 0x1e00..0x1e3f automatically. */
    u_int16 sdiFree;       /*0x1e1f SDI FIFO free in words. V4*/
    u_int16 audioFill;     /*0x1e20 Audio buffer fill in stereo samples. V4*/
    u_int16 reserved[4];   /*0x1e21..24 */
    u_int32 latestSOF;     /*0x1e25/1e26 latest start of frame V4 */
    u_int32 positionMsec;  /*0x1e27-28 play position if known. V3*/
    s_int16 resync;        /*0x1e29 > 0 for automatic m4a, ADIF, WMA resyncs*/
    /* 42 words */
    union { /* 22 available */
	u_int16 generic[22]; /*1e2a*/
	struct {
	    s_int16 txUartDiv;
	    s_int16 txUartByteSpeed;
	    u_int16 txPauseGpio;
	    s_int16 aecAdaptMultiplier; /*1e2d use 2 for default, and try 3..6 for higher speaker to mic gain. */
	    s_int16 reserved[14];
	    u_int16 channelMax[2]; /*for record level monitoring*/
	    u_int32 serialNumber; /*for Ogg Vorbis if enabled in WRAMADDR(11)*/
	} encoding;
	struct {
	    u_int32 curPacketSize;
	    u_int32 packetSize;
	} wma; /* 4*/
	struct {
	    u_int16 sceFoundMask; /*2a single-channel-el. found since last clr*/
	    u_int16 cpeFoundMask; /*2b channel-pair-el. found since last clr*/
	    u_int16 lfeFoundMask; /*2c low-frequency-el. found since last clr*/
	    u_int16 playSelect;   /*2d 0 = first any, initialized at aac init */
	    s_int16 dynCompress;  /*2e -8192=1.0, initialized at aac init */
	    s_int16 dynBoost;     /*2f  8192=1.0, initialized at aac init */
	    /*
	      playSelect:
	        0 = first sce or cpe or lfe
		xxxx0001 first sce
		xxxx0010 first cpe
		xxxx0011 first lfe
		eeee0101 sce eeee
		eeee0110 cpe eeee
		eeee0111 lfe eeee
	    */
	    /*
	      sbrAndPsStatus: (mask)
	        1 SBR present
		2 upsampling active
		4 PS present
		8 PS active
	     */
	    u_int16 sbrAndPsStatus; /*0x1e30 V3 */
#define AAC_SBR_PRESENT 1
#define AAC_UPSAMPLE_ACTIVE  2
#define AAC_PS_PRESENT 4
#define AAC_PS_ACTIVE  8
	    u_int16 sbrPsFlags; /*0x1e31 V4*/
	} aac; /* 3*/
	struct {
	    u_int32 bytesLeft;
	} midi; /* */
	struct {
	    s_int16 gain; /* 0x1e2a proposed gain offset, default = -12 */
	} vorbis;
    } i;
};
#endif/*V4*/

extern struct parametric parametric_x; /* 64 words */

/*
  midiConfig:
  1 1 1 1  1 1
  5 4 3 2  1 0 9 8  7 6 5 4  3 2 1 0
  - - - -  - - - -  - S S S  R R R R

  R = reverb   0 = auto (ON if clock>=3.0x)
               1 = off
	    2-15 = room size

  S = play speed  0 = 1x (normal speed)
                  1 = 2x, 2 = 4x, 3 = 8x, 4 = 16x, 5 = 32x, 6 = 64x, 7 = 128x
*/
#endif/*elseASM*/

#endif /* __PARAMETRIC_H__ */
