#ifndef __WAV_H__
#define __WAV_H__

#include <vstypes.h>

/** PCM format */
#define	WAVE_FORMAT_PCM			0x0001 
/** Basic ADPCM (not supported) */
#define	WAVE_FORMAT_ADPCM		0x0002
#define	WAVE_FORMAT_IEEE_FLOAT		0x0003
/** A-Law (not supported) */
#define	WAVE_FORMAT_ALAW		0x0006
/** Mu-Law (now supported) */
#define	WAVE_FORMAT_MULAW		0x0007
/** IMA ADPCM */
#define	WAVE_FORMAT_IMA_ADPCM		0x0011
/*See ftp://ftp.isi.edu/in-notes/rfc2361.txt */
#define	WAVE_FORMAT_EXTENSIBLE		0xfffe
#define WAVE_FORMAT_G721_ADPCM          0x0040
#define WAVE_FORMAT_MPEG                0x0050
#define WAVE_FORMAT_MPEGLAYER3          0x0055
//#define WAVE_FORMAT_G722_ADPCM          0x0065
#define WAVE_FORMAT_G722_ADPCM          0x028f

/*
  The WAVE_FORMAT_EXTENSIBLE format code indicates that there is an extension
  to the Format chunk. The extension has one field which declares the number
  of "valid" bits/sample (wValidBitsPerSample). Another field (dwChannelMask)
  contains a bits which indicate the mapping from channels to loudspeaker
  positions. The last field (SubFormat) is a 16-byte globally unique
  identifier (GUID).

  * With the WAVE_FORMAT_EXTENSIBLE format, the original bits/sample field
    (wBitsPerSample) must match the container size
    (8 * nBlockAlign / nChannels).
    This means that wBitsPerSample must be a multiple of 8. Reduced precision
    within the container size is now specified by wValidBitsPerSample.
  * The number of valid bits (wValidBitsPerSample) is informational only.
    The data is correctly represented in the precision of the container size.
    The number of valid bits can be any value from 1 to the container size in
    bits.
  * The loudspeaker position mask uses 18 bits, each bit corresponding to a
    speaker position (e.g. Front Left or Top Back Right), to indicate the
    channel to speaker mapping. More details are in the document cited above.
    This field is informational. An all-zero field indicates that channels are
    mapped to outputs in order: first channel to first output, second channel
    to second output, etc.
  * The first two bytes of the GUID form the sub-code specifying the data
    format code, e.g. WAVE_FORMAT_PCM. The remaining 14 bytes contain a fixed
    string, "\x00\x00\x00\x00\x10\x00\x80\x00\x00\xAA\x00\x38\x9B\x71".
*/

/** Riff 'f','m','t',' ' TAG format. This is a little-endian format. */
struct WavRiffFormatTag {
  u_int16 formatTag;		/**< WAV subformat */
  u_int16 channels;		/**< Number of audio channels */
  u_int32 samplesPerSecond;	/**< Sample rate -- mixed-endian! */
  u_int32 avgBytesPerSec;	/**< Average data rate */
  u_int16 blockAlign;		/**< Block align if format is block oriented */
  u_int16 bitsPerSample;	/**< Number of bits per audio sample */
#if 0
  /* start of extension */
  u_int16 cbSize;               /*2 Size of the extension: 22*/
  u_int16 wValidBitsPerSample;  /*2 at most 8 * M*/
  u_int32 dwChannelMask;        /*4 Speaker position mask: 0*/
  u_int16 SubFormat[8];         /*16 GUID (1st word is the data format code)*/
#endif
};

/* dwChannelMask bits: Channel order
   1. Front Left - FL
   2. Front Right - FR
   3. Front Center - FC
   4. Low Frequency - LF
   5. Back Left - BL
   6. Back Right - BR
   7. Front Left of Center - FLC
   8. Front Right of Center - FRC
   9. Back Center - BC
  10. Side Left - SL
  11. Side Right - SR
  12. Top Center - TC
  13. Top Front Left - TFL
  14. Top Front Center - TFC
  15. Top Front Right - TFR
  16. Top Back Left - TBL
  17. Top Back Center - TBC
  18. Top Back Right - TBR*/
#define SPEAKER_FRONT_LEFT             0x1
#define SPEAKER_FRONT_RIGHT            0x2
#define SPEAKER_FRONT_CENTER           0x4
#define SPEAKER_LOW_FREQUENCY          0x8
#define SPEAKER_BACK_LEFT              0x10
#define SPEAKER_BACK_RIGHT             0x20
#define SPEAKER_FRONT_LEFT_OF_CENTER   0x40
#define SPEAKER_FRONT_RIGHT_OF_CENTER  0x80
#define SPEAKER_BACK_CENTER            0x100
#define SPEAKER_SIDE_LEFT              0x200
#define SPEAKER_SIDE_RIGHT             0x400
#define SPEAKER_TOP_CENTER             0x800
#define SPEAKER_TOP_FRONT_LEFT         0x1000
#define SPEAKER_TOP_FRONT_CENTER       0x2000
#define SPEAKER_TOP_FRONT_RIGHT        0x4000
#define SPEAKER_TOP_BACK_LEFT          0x8000
#define SPEAKER_TOP_BACK_CENTER        0x10000
#define SPEAKER_TOP_BACK_RIGHT         0x20000
#define SPEAKER_RESERVED               0x80000000


#endif/*!__WAV_H__*/
