#ifndef STANDALONE_H
#define STANDALONE_H
#define STANDALONE  /*defined for vs1053 header files*/

/* Now support High-Capacity SD (SDHC) cards by default! */
/* 20180416 SCI control is now default for both player and recorder. */

#define UART_BUFFERED /* Use buffered UART commands. */
//#define USE_PRINTABLE_OUTPUT /*Use human-readable/decimal output.*/

/*
  There are three key-control alternatives:
   SCI_KEYS    - uses GPIO4 through GPIO6, can be used when SCI is in use.
   COMPAT_KEYS - use the three-button interface, but SCI cannot be connected at the same time.
   DIRECT_KEYS - read directly connected keys, but SCI cannot be connected at the same time.

  (The SCI control itself is always active in the player and recorder, but
   cannot be accessed if the SCI pins are used for keys.)
 */
//#define SCI_KEYS /* Both SCI control and keys (GPIO4 through GPIO6). */
#define COMPAT_KEYS  /**< Read the old 3-button interface of vs10xx prototyping board. (is overriden by SCI_KEYS) */
//#define DIRECT_KEYS /**< Read directly connected keys. (is overriden by COMPAT_KEYS and SCI_KEYS) */
//#define START_IN_FILEMODE /**< Start directly in filemode. */
//#define NO_FILEMODE /**< Disable filemode from UART control. */
//#define USE_INFO /**< Info command shows name and size of a file #xx. */

#define USE_EFI              /**< Try to find FAT from EFI partitions. */
#define USE_DASH_IN_PLAYER   /**< enable DASH container workaround for AAC. */
#define USE_DASH_IN_RECORDER /**< enable DASH container workaround for AAC. */
//#define USE_EARSPEAKER /**< With UART_BUFFERED adds 'e' to toggle EarSpeaker.*/
#define SHUFFLE_PLAY /**< shuffle play instead of random -- disabled in recorder if UART_BUFFERED */
#define SKIP_ID3V2   /**< quickly skip ID3v2 tags -- disabled in recorder if UART_BUFFERED */

//#define NO_DREQ_LED
//#define NO_WMA /**< reject WMA files */
//#define NO_AAC /**< reject AAC files */
//#define NO_MP3 /**< do not play mp3 */

//#define ENABLE_I2S /**< Enable I2S output*/
#define ENABLE_HIGHREF /**< Use 1.6V reference voltage -- use if AVDD >= 3.3V */

/*Note: these have not been tested with the VSIDE solution. */
//#define SAVE_VOLUME /**< Save volume setting to EEPROM */
//#define SAVE_POSITION /**< Save playing song to EEPROM */

//#define PAUSE_AT_POWERON /* if PAUSE_BEFORE_PLAY at poweron, pause before the first file and change mode to PAUSE_AFTER_PLAY. */

#ifdef UART_BUFFERED
#define CLOCKF_VAL 0xe000 //should not allow clock change with UART control
#else
#define CLOCKF_VAL 0xb000 //set the suitable clock 4.0x+1.5x
#endif
#define VOL_VAL    0x2020 //start with -16dB volume
#define BASS_VAL   0x33d9 //loudness value, lowest bit must be 1

#define RECORD_FS  48000U //24000 /* The default recording samplerate. */
//#define NO_ENCODE_MP3 //If not encoding mp3 save instruction memory (339).

//#define SD_POWER_ENABLE (1<<7)
#define SD_POWER_ENABLE 0 //not active


/****************************************************************************/
/*
  Definitions for AICTRL3
 */
/*
  15: CODECMODE
  14: ENABLEAEC
  13: ENABLEUART
  12: -               | record_on
  11: PAUSEON         | pauseon
  10: NORIFF
  9: -
  8: -
  7-4: encodemode     | by_name, at_end, no_numfiles, iterate
  3: -                | file ready
  2-0: channelmode    | playmode, random
 */
#define CTRL3_RECORD_ON         (1<<12)
#define CTRL3_PAUSE_ON          (1<<11) /* Both play and record pause */
#define CTRL3_BY_NAME           (1<<7)  /*locate by name*/
#define CTRL3_AT_END            (1<<6)
#define CTRL3_NO_NUMFILES       (1<<5)  /*comment out to disable the feature*/
#define CTRL3_ITERATE           (1<<4)  /*used for directory listing - calls IterateFilesCallback() */
#define CTRL3_FILE_READY        (1<<3)

#define CTRL3_PLAY_MODE_MASK    (3<<1)
#define CTRL3_PAUSE_AFTER_PLAY  (3<<1)
#define CTRL3_PAUSE_BEFORE_PLAY (2<<1)
#define CTRL3_LOOP_SONG         (1<<1)
#define CTRL3_PLAY_NEXT         (0<<1)

#define CTRL3_RANDOM_PLAY       (1<<0) /* SCI_AICTRL3 */



#define         MMC_GO_IDLE_STATE   0
#define         MMC_SEND_OP_COND   1
#define         MMC_SEND_IF_COND   8
#define         MMC_SEND_CSD   9
#define         MMC_SEND_CID   10
#define MMC_STOP_TRANSMISSION   12
#define         MMC_SEND_STATUS   13
#define         MMC_SET_BLOCKLEN   16
#define         MMC_READ_SINGLE_BLOCK   17
#define         MMC_READ_MULTIPLE_BLOCK   18
#define         MMC_WRITE_BLOCK   24
#define         MMC_PROGRAM_CSD   27
#define         MMC_SET_WRITE_PROT   28
#define         MMC_CLR_WRITE_PROT   29
#define         MMC_SEND_WRITE_PROT   30
#define         MMC_TAG_SECTOR_START   32
#define         MMC_TAG_SECTOR_END   33
#define         MMC_UNTAG_SECTOR   34
#define         MMC_TAG_ERASE_GROUP_START   35
#define         MMC_TAG_ERARE_GROUP_END   36
#define         MMC_UNTAG_ERASE_GROUP   37
#define         MMC_ERASE   38
#define         MMC_READ_OCR     58
#define         MMC_CRC_ON_OFF   59
#define         MMC_R1_BUSY   0x80
#define         MMC_R1_PARAMETER   0x40
#define         MMC_R1_ADDRESS   0x20
#define         MMC_R1_ERASE_SEQ   0x10
#define         MMC_R1_COM_CRC   0x08
#define         MMC_R1_ILLEGAL_COM   0x04
#define         MMC_R1_ERASE_RESET   0x02
#define         MMC_R1_IDLE_STATE   0x01
#define         MMC_STARTBLOCK_READ   0xFE
#define         MMC_STARTBLOCK_WRITE   0xFE
#define         MMC_STARTBLOCK_MWRITE   0xFC
#define         MMC_STOPTRAN_WRITE   0xFD
#define         MMC_DE_MASK   0x1F
#define         MMC_DE_ERROR   0x01
#define         MMC_DE_CC_ERROR   0x02
#define         MMC_DE_ECC_FAIL   0x04
#define         MMC_DE_OUT_OF_RANGE   0x04
#define         MMC_DE_CARD_LOCKED   0x04
#define         MMC_DR_MASK   0x1F
#define         MMC_DR_ACCEPT   0x05
#define         MMC_DR_REJECT_CRC   0x0B
#define         MMC_DR_REJECT_WRITE_ERROR   0x0D

#ifdef ASM
/* asm-only definitions here */


#else/*ASM*/
/* C-only definitions here */

#include <h1063/vsmpg.h>
#include <vs1063.h>
#include <h1063/common.h>
#include <h1063/stream.h>
#include <h1063/layers.h>
#include <h1063/asmfuncs.h>
#include <h1063/parametric.h>
#include <vstypes.h>

#ifdef RECORDER
#include <h1063/encoders.h>
#endif

auto s_int16 myrand(void); /* output 0..0x3fff */
extern __y s_int16 rand_seed;

#include "fat.h"

extern s_int16 hcShift;

/* Low-level UART output functions. */
void putch(register __a0 int ch);
void putword(register __a0 int ch);

void putstrp(register __i2 u_int16 *p);
void putstrpy(register __i2 __mem_y u_int16 *p);
void putdec(register __reg_b u_int32 a);
void puthex(register __c0 u_int16 d);

void BusyWait10(void);

void SpiLoad(register __i2 short startAddr);

void MyReset();

auto void SpiSendClocks(void);
auto u_int16 SpiSendReceiveMmc(register __a0 u_int16 topAlignedData,
			       register __a1 numBits);
auto u_int16 MmcCommand(register __b0 s_int16 cmd, register __d u_int32 arg);
s_int16 InitializeMmc(s_int16 tries);
s_int16 InitializeMmcRecorder(s_int16 tries);
auto void ReadDiskSectorTo(register __a u_int32 sector,
			   register __i2 u_int16 *buf);
auto void WritePhysicalSectorY(register __d u_int32 address,
			      register __i3 __y u_int16 *dataBufPtr);
auto u_int16 GetByte(register __c0 u_int16 n);
auto u_int16 GetWord(register __c0 u_int16 n);
auto u_int32 GetLong(register __c0 u_int16 n);
auto u_int16 InitFileSystem(void);
auto __y struct FRAGMENT *FragmentList(register __i2 __y struct FRAGMENT *frag,
			  register __b u_int32 fatCluster);
auto s_int16 HandleDir(register __i2 __y struct FRAGMENT *curFragment,
		       __y struct FRAGMENT *nextFragment);
auto s_int16 OpenFile(register __c0 u_int16 fileNum);
auto s_int16 CheckFileType(register __a u_int32 name);
auto s_int16 CheckFileType1063(register __a u_int32 name);

u_int16 SciModeOr(void);

extern __mem_y u_int16 fileName[6];
#ifdef FATINFO_IN_Y
extern __mem_y struct FATINFO fatInfo;
#else
extern struct FATINFO fatInfo;
#endif
void IterateFilesCallback(register __b0 u_int16 *name);

extern u_int16 created[3]; /*create date, time, 100ths, defaults to 0 */

int UartFill(void);
int UartGetByte(void);
int UartPeekByte(void);

u_int16 Shuffle(register __c0 u_int16 files, register __c1 u_int16 old);

u_int16 GetI6(void);
void CallEncoders(void); /*patched Encoders()*/
void TimerInt(void);

void puthex2(u_int16 d);
void Monitor(void);
void IdleHook(void);
auto u_int32 NextSector(void);
auto u_int16 swap(register __a0 u_int16 val);
auto s_int32 MmcStopTransmission(void); //dummy return value

extern u_int16 __y extClock4KHz, clockX, clockA, clockMaxA, oldClock4KHz;
void AacPatch(); //Only DASH support
#endif/*elseASM*/

#endif/*STANDALONE_H*/
